using Nirvana.Common;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Waste.Domain;

namespace Waste.Application
{
    /// <summary>
    /// 商户授权处理
    /// </summary>
    public interface IBusinessApiService
    {
        /// <summary>
        /// 授权列表
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        Task<PageParms<BusinessApiInfo>> GetListAsync(QueryParams param);
        /// <summary>
        /// 信息提交
        /// </summary>
        /// <param name="role"></param>
        /// <returns></returns>
        Task<ResultInfo> SubmitFormAsync(W_BusinessAppApi role);
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="keyValue"></param>
        /// <returns></returns>
        Task<ResultInfo> DeleteFormAsync(Guid keyValue);
        /// <summary>
        /// 详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<W_BusinessAppApi> DetailAsync(Guid id);
        /// <summary>
        /// 获取推送列表
        /// </summary>
        /// <returns></returns>
        Task<List<W_BusinessPush>> GetPushListAsync();
        /// <summary>
        /// 检查是否在IP白名单中
        /// </summary>
        /// <param name="appid"></param>
        /// <param name="ip"></param>
        /// <returns></returns>
        Task<ResultInfo> IsWhiteIP(string appid, string ip);
        /// <summary>
        /// 增加推送消息
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        Task InsertPushInfoAsync(W_BusinessPush data);
    }
}
